@@ -81,6 +81,9 @@ class SafeElement(GObject.Object):
     sensitive = GObject.Property(type=bool, default=True)
     sorted_handler_id: int | None = None
 
+    _notes = None
+    _name = None
+
     def __init__(self, db_manager: DatabaseManager, element: Entry | Group):
         """GObject to handle a safe element.
 
@@ -96,18 +99,15 @@ class SafeElement(GObject.Object):
         self.is_group = isinstance(self, SafeGroup)
         self.is_entry = isinstance(self, SafeEntry)
 
-        self._notes: str = element.notes or ""
-        if self.is_group:
-            self._name = element.name or ""
-        else:
-            self._name = element.title or ""
-
     def __eq__(self, other):  # pylint: disable=arguments-differ
         if isinstance(other, SafeElement):
             return self.uuid == other.uuid
 
         return False
 
+    def __hash__(self):  # pylint: disable=arguments-differ
+        return hash(self.uuid)
+
     @GObject.Signal(flags=GObject.SignalFlags.ACTION)
     def updated(self):
         """Update the entry.
@@ -241,6 +241,12 @@ class SafeElement(GObject.Object):
         :returns: name or an empty string if there is none
         :rtype: str
         """
+        if self._name is None:
+            if self.is_group:
+                self._name = self._element.name or ""
+            else:
+                self._name = self._element.title or ""
+
         return self._name
 
     @name.setter  # type: ignore
@@ -276,6 +282,9 @@ class SafeElement(GObject.Object):
         :returns: notes or an empty string if there is none
         :rtype: str
         """
+        if self._notes is None:
+            self._notes = self._element.notes or ""
+
         return self._notes
 
     @notes.setter  # type: ignore
@@ -344,6 +353,18 @@ class SafeElement(GObject.Object):
         """
         return self._element.uuid
 
+    @GObject.Property(type=str, flags=GObject.ParamFlags.READABLE)
+    def parent_uuid_str(self) -> str:
+        """UUID of the parent group of the element.
+
+        :returns: uuid of the parent
+        :rtype: UUID
+        """
+        if self.is_root_group:
+            return ""
+
+        return str(self.parentgroup_uuid)
+
     @property
     def path(self) -> list[str]:
         return self._element.path
@@ -375,6 +396,25 @@ class SafeElement(GObject.Object):
             time.second,
         )
 
+    @GObject.Property(type=int, flags=GObject.ParamFlags.READABLE)
+    def ctime_int(self) -> int:
+        try:
+            time = self._element.ctime
+        except ValueError:
+            logging.exception("Invalid creation time")
+            return -1
+        except OverflowError:
+            logging.exception("Creation time for %s overflows", self.name)
+            return -1
+        except Exception:  # pylint: disable=broad-except
+            logging.exception("Could not read creation time")
+            return -1
+
+        if not time:
+            return -1
+
+        return time.timestamp()
+
     @property
     def ctime(self) -> GLib.DateTime | None:
         """The UTC creation time of the element."""
@@ -446,8 +486,6 @@ class SafeGroup(SafeElement):
         """
         super().__init__(db_manager, group)
 
-        self._group: Group = group
-
         if self.is_root_group:
             self._db_manager.root = self
 
@@ -531,14 +569,22 @@ class SafeGroup(SafeElement):
             self._subgroups_filter.changed(Gtk.FilterChange.DIFFERENT)  # type: ignore
 
     def init_subgroups(self):
-        self._subgroups_filter = Gtk.CustomFilter.new(self._group_filter_func)
+        expr = Gtk.PropertyExpression.new(
+            SafeElement.__gtype__, None, "parent_uuid_str"
+        )
+        self._subgroups_filter = Gtk.StringFilter.new(expr)
+        self._subgroups_filter.props.search = str(self.uuid)
         self._subgroups = Gtk.FilterListModel.new(
             self._db_manager.groups,
             self._subgroups_filter,
         )
 
     def init_entries(self):
-        self._entries_filter = Gtk.CustomFilter.new(self._group_filter_func)
+        expr = Gtk.PropertyExpression.new(
+            SafeElement.__gtype__, None, "parent_uuid_str"
+        )
+        self._entries_filter = Gtk.StringFilter.new(expr)
+        self._entries_filter.props.search = str(self.uuid)
         self._entries = Gtk.FilterListModel.new(
             self._db_manager.entries,
             self._entries_filter,
@@ -561,16 +607,7 @@ class SafeGroup(SafeElement):
     @property
     def group(self) -> Group:
         """Returns the private pykeepass group."""
-        return self._group
-
-    def _entry_filter_func(self, entry: SafeEntry) -> bool:
-        return entry.parentgroup_uuid == self.uuid
-
-    def _group_filter_func(self, group: SafeGroup) -> bool:
-        if group.is_root_group:
-            return False
-
-        return group.parentgroup_uuid == self.uuid
+        return self._element
 
     def emit_children_changed(self, removed, added):
         self.emit(self.children_changed, removed, added)
@@ -584,6 +621,13 @@ class SafeEntry(SafeElement):
     _note_key = "Notes"
     _otp: TOTP | None = None
     _otp_key = "otp"
+    _attributes: AttributesModel | None = None
+    _attachments: list[Attachment] | None = None
+    _username: str | None = None
+    _url: str | None = None
+    _password: str | None = None
+    _color: EntryColor | None = None
+    _icon_nr: str | None = None
 
     history_saved = GObject.Signal()
 
@@ -595,42 +639,6 @@ class SafeEntry(SafeElement):
         """
         super().__init__(db_manager, entry)
 
-        self._entry: Entry = entry
-
-        self._attachments: list[Attachment] = entry.attachments or []
-
-        # NOTE Can fail at libpykeepass, see
-        # https://github.com/libkeepass/pykeepass/issues/254
-        # TODO Read as many attributes as possible until we see an error.
-        try:
-            attributes = {
-                key: value
-                for key, value in entry.custom_properties.items()
-                if key not in (self._color_key, self._note_key, self._otp_key)
-            }
-        except Exception:  # pylint: disable=broad-except
-            logging.exception("Could not read attributes")
-            attributes = {}
-
-        self._attributes = AttributesModel(attributes)
-
-        color_value: str = entry.get_custom_property(self._color_key)
-        self._color: str = color_value or EntryColor.NONE.value
-
-        self._icon_nr: str = entry.icon or ""
-        self._password: str = entry.password or ""
-        self._url: str = entry.url or ""
-        self._username: str = entry.username or ""
-
-        if otp_uri := entry.otp:
-            try:
-                if otp_uri.startswith("otpauth://"):
-                    self._otp = parse_uri(otp_uri)  # type: ignore
-                else:
-                    self._otp = TOTP(otp_uri)
-            except ValueError:
-                logging.exception("Could not parse OTP")
-
         self._check_expiration()
 
     @property
@@ -640,7 +648,7 @@ class SafeEntry(SafeElement):
         :returns: entry
         :rtype: Entry
         """
-        return self._entry
+        return self._element
 
     def duplicate(self) -> None:
         """Duplicate an entry."""
@@ -708,18 +716,21 @@ class SafeEntry(SafeElement):
     def save_history(self) -> None:
         """Save current version of the entry in its history."""
         # NOTE Attachments are references, so duplicating them is ok.
-        self._entry.save_history()
+        self._element.save_history()
         self.updated()
         self.emit(self.history_saved)
 
     def delete_history(self, entry: SafeEntry) -> None:
         """Delete entry from the history of self."""
-        self._entry.delete_history(entry.entry)
+        self._element.delete_history(entry.entry)
         self.updated()
         self.emit(self.history_saved)
 
     @GObject.Property(type=object, flags=GObject.ParamFlags.READABLE)
     def attachments(self) -> list[Attachment]:
+        if self._attachments is None:
+            self._attachments: list[Attachment] = self._element.attachments or []
+
         return self._attachments
 
     def add_attachment(self, byte_buffer: bytes, filename: str) -> Attachment:
@@ -730,8 +741,10 @@ class SafeEntry(SafeElement):
         :returns: attachment
         :rtype: Attachment
         """
+        assert self._attachments is not None
+
         attachment_id = self._db_manager.db.add_binary(byte_buffer)
-        attachment = self._entry.add_attachment(attachment_id, filename)
+        attachment = self._element.add_attachment(attachment_id, filename)
         self._attachments.append(attachment)
         self.updated()
         self.notify("attachments")
@@ -743,6 +756,8 @@ class SafeEntry(SafeElement):
 
         :param Attachmennt attachment: attachment to delete
         """
+        assert self._attachments is not None
+
         self._db_manager.db.delete_binary(attachment.id)
         self._attachments.remove(attachment)
         self.notify("attachments")
@@ -755,6 +770,8 @@ class SafeEntry(SafeElement):
         :returns: attachment
         :rtype: Attachment
         """
+        assert self._attachments is not None
+
         for attachment in self._attachments:
             if str(attachment.id) == id_:
                 return attachment
@@ -770,6 +787,22 @@ class SafeEntry(SafeElement):
 
     @GObject.Property(type=object, flags=GObject.ParamFlags.READABLE)
     def attributes(self) -> AttributesModel:
+        if self._attributes is None:
+            # NOTE Can fail at libpykeepass, see
+            # https://github.com/libkeepass/pykeepass/issues/254
+            # TODO Read as many attributes as possible until we see an error.
+            try:
+                attributes = {
+                    key: value
+                    for key, value in self._element.custom_properties.items()
+                    if key not in (self._color_key, self._note_key, self._otp_key)
+                }
+            except Exception:  # pylint: disable=broad-except
+                logging.exception("Could not read attributes")
+                attributes = {}
+
+            self._attributes = AttributesModel(attributes)
+
         return self._attributes
 
     def has_attribute(self, key: str) -> bool:
@@ -777,6 +810,8 @@ class SafeEntry(SafeElement):
 
         :param str key: attribute key to check
         """
+        assert self._attributes is not None
+
         return self._attributes.has_attribute(key)
 
     def set_attribute(self, key: str, value: str, protected: bool = False) -> None:
@@ -785,15 +820,17 @@ class SafeEntry(SafeElement):
         :param str key: attribute key
         :param str value: attribute value
         """
+        assert self._attributes is not None
+
         if self.props.attributes.get(key) == value:
             return
 
-        if self._entry.get_custom_property(
+        if self._element.get_custom_property(
             key,
-        ) == value and protected == self._entry.is_custom_property_protected(key):
+        ) == value and protected == self._element.is_custom_property_protected(key):
             return
 
-        self._entry.set_custom_property(key, value, protect=protected)
+        self._element.set_custom_property(key, value, protect=protected)
 
         self._attributes.insert(key, value)
 
@@ -805,10 +842,12 @@ class SafeEntry(SafeElement):
 
         :param key: attribute key to delete
         """
+        assert self._attributes is not None
+
         if not self.has_attribute(key):
             return
 
-        self._entry.delete_custom_property(key)
+        self._element.delete_custom_property(key)
         self._attributes.pop(key)
         self.updated()
         self.notify("attributes")
@@ -827,6 +866,10 @@ class SafeEntry(SafeElement):
         :returns: color as string
         :rtype: str
         """
+        if self._color is None:
+            color_value: str = self._element.get_custom_property(self._color_key)
+            self._color: str = color_value or EntryColor.NONE.value
+
         return self._color
 
     @color.setter  # type: ignore
@@ -837,12 +880,12 @@ class SafeEntry(SafeElement):
         """
         if new_color != self._color:
             self._color = new_color
-            self._entry.set_custom_property(self._color_key, new_color)
+            self._element.set_custom_property(self._color_key, new_color)
             self.updated()
 
     @property
     def history(self) -> list[SafeEntry]:
-        history = self._entry.history
+        history = self._element.history
         return [SafeEntry(self._db_manager, entry) for entry in history]
 
     @GObject.Property(type=object)
@@ -852,6 +895,9 @@ class SafeEntry(SafeElement):
         :returns: icon number or "0" if no icon
         :rtype: str
         """
+        if self._icon_nr is None:
+            self._icon_nr: str = self._element.icon or ""
+
         try:
             return ICONS[self._icon_nr]
         except KeyError:
@@ -865,7 +911,7 @@ class SafeEntry(SafeElement):
         """
         if new_icon_nr != self._icon_nr:
             self._icon_nr = new_icon_nr
-            self._entry.icon = new_icon_nr
+            self._element.icon = new_icon_nr
             self.notify("icon-name")
             self.updated()
 
@@ -889,6 +935,19 @@ class SafeEntry(SafeElement):
 
     @GObject.Property(type=str, default="")
     def otp(self) -> str:
+        if self._otp is None:
+            if otp_uri := self._element.otp:
+                try:
+                    if otp_uri.startswith("otpauth://"):
+                        self._otp = parse_uri(otp_uri)  # type: ignore
+                    else:
+                        self._otp = TOTP(otp_uri)
+                except ValueError:
+                    logging.exception("Could not parse OTP")
+                    self._otp = ""
+            else:
+                self._otp = ""
+
         if self._otp:
             return self._otp.secret
 
@@ -957,6 +1016,9 @@ class SafeEntry(SafeElement):
         :returns: password or an empty string if there is none
         :rtype: str
         """
+        if self._password is None:
+            self._password: str = self._element.password or ""
+
         return self._password
 
     @password.setter  # type: ignore
@@ -967,7 +1029,7 @@ class SafeEntry(SafeElement):
         """
         if new_password != self._password:
             self._password = new_password
-            self._entry.password = new_password
+            self._element.password = new_password
             self.updated()
 
     @GObject.Property(type=str, default="")
@@ -977,6 +1039,9 @@ class SafeEntry(SafeElement):
         :returns: url or an empty string if there is none
         :rtype: str
         """
+        if self._url is None:
+            self._url: str = self._element.url or ""
+
         return self._url
 
     @url.setter  # type: ignore
@@ -987,7 +1052,7 @@ class SafeEntry(SafeElement):
         """
         if new_url != self._url:
             self._url = new_url
-            self._entry.url = new_url
+            self._element.url = new_url
             self.updated()
 
     @GObject.Property(type=str, default="")
@@ -997,6 +1062,9 @@ class SafeEntry(SafeElement):
         :returns: username or an empty string if there is none
         :rtype: str
         """
+        if self._username is None:
+            self._username: str = self._element.username or ""
+
         return self._username
 
     @username.setter  # type: ignore
@@ -1007,7 +1075,7 @@ class SafeEntry(SafeElement):
         """
         if new_username != self._username:
             self._username = new_username
-            self._entry.username = new_username
+            self._element.username = new_username
             self.updated()
 
     @GObject.Property(type=bool, default=False)
@@ -1111,7 +1179,7 @@ ICONS = {
     "17": Icon("media-optical-cd-audio-symbolic"),
     "18": Icon("video-display-symbolic"),
     "19": Icon("mail-unread-symbolic", True),
-    "20": Icon("emblem-system-symbolic"),
+    "20": Icon("settings-symbolic"),
     "21": Icon("edit-paste-symbolic"),
     "22": Icon("edit-paste-symbolic"),
     "23": Icon("display-with-window-symbolic", True),
